/*************************************************************************
 *  This is the operative scan source code for a CPMG Scan.  The prototype for
 *  calling this function is as follows:
 *
 *  CpmgScan(SF, SW, P2_time, P2_after, tau, echo_points, echo_loops, 
 *  repetition_delay, num_scans, bypass_fir, adc_freq, blankingBit,
 *  amp, errorReturn);
 *
 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "spinapi.h"
#include "mex.h"

#define BLANKING_DELAY 3.0

int main(double SF, double SW, double P2_time, double P2_phase, double P2_after, double tau, 
    int echo_points, int echo_loops, double repetition_delay, int num_scans, 
    int bypass_fir, double adc_freq, int blankingBit, double amp, double blank_delay, double P1_time,
    int* errorReturn)
{
    int *real,*imag;
    int start;
    int echo_loop_label;
    int scan_loop_label;              
    int cmd;
    int do_top_trigger;
    int dec_amount;
	int num_points;
    int blankPA, flagBits = 0 ;		//Bit shift left to set blanking bits    
    double P1_phase;
    double P1_after1;
    double P1_after;
    double top_time;
    double actual_SW;
    
    char txt_fname[1024];
    char fid_fname[1024];
    char jcamp_fname[1024];

    PB_OVERFLOW_STRUCT of;  
    
    printf ("SF               : %.4lff MHz\n", SF);
    printf ("SW               : %f kHz\n", SW);
    printf ("P1_time          : %f us\n",  P1_time);
    printf ("P2_time          : %f us\n",  P2_time);
    printf ("P2_after         : %f us\n", P2_after);
    printf ("tau              : %f us\n", tau);
    printf ("echo_points      : %d\n", echo_points);
    printf ("echo_loops       : %d\n", echo_loops);
    printf ("repetition_delay : %.2lf s\n", repetition_delay);
    printf ("num_scans        : %d\n", num_scans);
    printf ("bypass fir       : %d\n", bypass_fir);
    printf ("adc_freq         : %.2lf MHz\n",adc_freq);

    P1_phase = P2_phase + 90;
    P1_phase -= 360.0*floor(P1_phase/360.0);
    
    
    if (pb_count_boards () <= 0)
    {
        printf("No RadioProcessor boards were detected in your system.\n");
        errorReturn[0] = -1;
        return -1;
    }
    
    printf ("Using spinapi library version %s\n", pb_get_version());
    
    if(pb_init()) {
        printf ("Error initializing board: %s\n", pb_get_error());    
        errorReturn[0] = -2;
		return -1;
    }
    if(blankingBit>5)
    {
		printf("Blanking bit cannot greater than 5.\n");
		errorReturn[0] = -3;
		return -1;	
	}
	flagBits= (1<<blankingBit)&0x3F;
    pb_set_defaults();
    pb_core_clock(adc_freq);

    pb_overflow(1,0); // reset the overflow counters
    pb_scan_count(1); // reset scan counter

    ///
    /// Program frequency and phase registers
    ///
    pb_start_programming(FREQ_REGS);
    pb_set_freq(SF);
    pb_stop_programming();

    pb_start_programming(COS_PHASE_REGS);
    pb_set_phase(0.0);
    pb_set_phase(90.0);
    pb_set_phase(180.0);
    pb_set_phase(270.0);
    pb_stop_programming();

    pb_start_programming(SIN_PHASE_REGS);
    pb_set_phase(0.0);
    pb_set_phase(90.0);
    pb_set_phase(180.0);
    pb_set_phase(270.0);
    pb_stop_programming();

    pb_start_programming(TX_PHASE_REGS);
    pb_set_phase(0);
    pb_set_phase(P1_phase);
    pb_set_phase(P2_phase);
    pb_stop_programming();
    
    pb_set_amp(amp,0);

// User friendly names for the phase registers of the cos and sin channels
#define PHASE000 0
#define PHASE090 1
#define PHASE180 2
#define PHASE270 3

// User friendly names for the control bits
#define TX_ENABLE 1
#define TX_DISABLE 0
#define PHASE_RESET 1
#define NO_PHASE_RESET 0
#define DO_TRIGGER 1
#define NO_TRIGGER 0
#define P1_PHASE 1
#define P2_PHASE 2

    ///
    /// Set acquisition parameters
    ///
    if (bypass_fir) {
        cmd = BYPASS_FIR;   
    }
    dec_amount = pb_setup_filters(SW/1000.0, num_scans, cmd);
     
    actual_SW = (adc_freq*1.0e6)/(double)dec_amount;
    printf ("Actual SW used is %f kHz (desired was %f kHz)\n", actual_SW/1000.0, SW);

    top_time = ((double)echo_points)/(actual_SW/1.0e6); // top time in us

    P1_after = tau - top_time / 2.0;
	
    if (echo_points == 0) {
		num_points = floor( ((tau + echo_loops*(P1_time + 2.0*tau))/1.0e6) * actual_SW);
        pb_set_num_points(num_points);
        pb_set_scan_segments(1);
        do_top_trigger = NO_TRIGGER;
        blankPA = flagBits;  //since data is acquired continuously we cannot blank the PA between pulses
        printf ("Acquiring data in a single, continuous scan. Power Amplifier enabled during acquisition\n");
    }
    else {
        pb_set_num_points(echo_points);
        pb_set_scan_segments(echo_loops + 1); // +1 because 1st segment
        do_top_trigger = DO_TRIGGER;
        printf ("Acquiring only the tops of the echos\n");
        if(tau-top_time > blank_delay*1000.0){
		  blankPA = 0;//if we have > blank_delay ms between end of acquisition and start of next pulse, blank PA during acquisition
          printf("Time between acquisition and P1 pulse %lf > %lf (Blanking Delay).\n\tBlanking Power Amplifier during acquisition\n", (tau-top_time), (blank_delay*1000));
        }
	    else {
	      blankPA = flagBits; //otherwise leave the PA enabled during the acquisition
          printf("Time between acquisition and P1 pulse %lf < %lf (Blanking Delay).\n\tPower Amplifier enabled during acquisition\n",(tau-top_time), (blank_delay*1000));
        }  
    }

    ///
    /// Specify pulse program
    ///

//pb_inst_radio_shape (int freq, int cos_phase, int sin_phase, int tx_phase,
//		     int tx_enable, int phase_reset, int trigger_scan,
//		     int use_shape, int amp, int flags, int inst,
//		     int inst_data, double length)

    pb_start_programming(PULSE_PROGRAM);

	// Reset phase initially, so that the phase of the excitation pulse will be
	// the same for every scan. This is the beginning of the scan loop
    scan_loop_label =
    // 3 ms of Blanking delay is added here
    pb_inst_radio_shape(0,PHASE090, PHASE000, 0, TX_DISABLE, PHASE_RESET, NO_TRIGGER, 0, 0,flagBits,
                   LOOP, num_scans, blank_delay*ms);

	// 90 degree pulse (P2)
    pb_inst_radio_shape(0,PHASE090, PHASE000, P2_PHASE, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER,0,0,flagBits,
                   CONTINUE, 0, P2_time * us);
    // wait for for the transient to subside...
    pb_inst_radio_shape(0,PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, NO_TRIGGER, 0,0,
				   (blankPA & flagBits), CONTINUE, 0, P2_after * us);
    // ... then trigger the acquisition. 
    if(top_time != 0) {
		pb_inst_radio_shape(0,PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, DO_TRIGGER,0,0,
				   (blankPA & flagBits), CONTINUE, 0, top_time * us);
	}
	pb_inst_radio_shape(0,PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, DO_TRIGGER,0,0,
				   flagBits, CONTINUE, 0, (tau - top_time) * us);

    // These next three instructions will loop echo_loops times. 
    // 180 degree pulse (P1)    
    echo_loop_label =
    pb_inst_radio_shape(0,PHASE090, PHASE000, P1_PHASE, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0,0,flagBits,
                   LOOP, echo_loops, P1_time*us);
    // wait through half of the echo ...
    pb_inst_radio_shape(0, PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, NO_TRIGGER,0,0,
				(blankPA & flagBits), CONTINUE, 0, P1_after*us);
    // ... then trigger acquisition and wait through the other half. (If this is to be
    // a singlue continuous acquisition, do_top_trigger is 0 so a trigger will not
    // occur here)
    if(top_time != 0) {
	pb_inst_radio_shape(0, PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, do_top_trigger,0,0, 
				(blankPA & flagBits), CONTINUE, 0, top_time*us);
	}
	pb_inst_radio_shape(0, PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, do_top_trigger,0,0, flagBits,
                  END_LOOP, echo_loop_label, P1_after*us);

    // Allow sample to relax before acuiring another scan
    pb_inst_radio_shape(0,PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, NO_TRIGGER,0,0, 0x00,
                   CONTINUE, 0, repetition_delay*1000*ms);

    // Loop back and do scan again. This will occur num_scans times
    pb_inst_radio_shape(0,PHASE090, PHASE000, 0, TX_DISABLE, PHASE_RESET, NO_TRIGGER,0,0, 0x00,
                   END_LOOP, scan_loop_label, 1.0*us);
    
    // Then stop the pulse program
    pb_inst_radio_shape(0,PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, NO_TRIGGER,0,0, 0x00,
                   STOP, 0, 1.0*us);
    
    pb_stop_programming();

    ///
    /// Trigger pulse program
    ///
    pb_start();

    pb_close();
    errorReturn[0] = 0;
		printf("\n\n");
    return 0;
}



void mexFunction(int nlhs, mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
    double SF;
    double SW;
    double P2_time;
    double P1_time;
    double P2_phase;
    double P2_after;
    double tau;
    int echo_points;
    int echo_loops;
    double repetition_delay;
    int num_scans;
    int bypass_fir;
    double adc_freq;
  	int blankingBit;
    double amp;
    double blank_delay;
  	int *errorReturn;
  	
    SF = mxGetScalar(prhs[0]);
    SW = mxGetScalar(prhs[1]);
    P2_time = mxGetScalar(prhs[2]);
    P2_phase = mxGetScalar(prhs[3]);
    P2_after = mxGetScalar(prhs[4]);
    tau = mxGetScalar(prhs[5]);
    echo_points = mxGetScalar(prhs[6]);
    echo_loops = mxGetScalar(prhs[7]);
    repetition_delay = mxGetScalar(prhs[8]);
    num_scans = mxGetScalar(prhs[9]);
    bypass_fir = mxGetScalar(prhs[10]);
    adc_freq = mxGetScalar(prhs[11]);
    blankingBit = mxGetScalar(prhs[12]);
    amp = mxGetScalar(prhs[13]);
    blank_delay = (mxGetScalar(prhs[14]))/1000.0; //expected in ms, not us
    P1_time = mxGetScalar(prhs[15]);
    
    plhs[0] = mxCreateNumericMatrix(1,1, mxINT32_CLASS, mxREAL);
    errorReturn=(int*)mxGetPr(plhs[0]);
    
    
    main(SF, SW, P2_time, P2_phase, P2_after, tau, echo_points, echo_loops, 
    repetition_delay, num_scans, bypass_fir, adc_freq, blankingBit, amp,
    blank_delay, P1_time, errorReturn);
    
}
