/* This scan program differs in that it runs just long enough to start the
 * scan and then returns control to MATLAB.  Downloading and saving the 
 * data is left up to the pbGetDataPlus function. */

#include "math.h"
#include "mex.h"
#include "spinapi.h"
#include "string.h"
#include "stdio.h"
#include "stdlib.h"
#define RAM_SIZE (16*1024)

double pi = 3.1415926;

void shape_make_sinc(float *shape_data, int lobes)
{
    int i;

    double x;
    double scale = (double)lobes*(2.0*pi);
    for(i=0; i < 1024; i++) {
        x = (double)(i-512)*scale/1024.0;
        if((x) == 0.0) {
            shape_data[i] = 1.0;
        }
        else
            shape_data[i] = sin(x)/x;
    }   
}


int main(int num_points, double SF, double SW, double pulse_time, 
double trans_time, double repetition_delay, int num_scans,
double tx_phase, int bypass_fir, double adc_freq, float amp_val, char *shape)
{
    int start;
    PB_OVERFLOW_STRUCT of;

    // Pulse program parameters (values are set by command line parameters)
    
    double rxSF = SF;
    double wait_time;
	float shape_data[1024];

    // Variables used within program
    int cmd;
    int n = 0;
    int dec_amount;
    double actual_SW;
    int status;
  
    if (num_points > RAM_SIZE || num_points < 1) {
        printf ("Error: Maximum number of points is %d\n", RAM_SIZE);
		return -1;    
    }
    if (num_scans < 1) {
        printf ("Error: there must be at least one scan\n");
		return -1;    
    }
    if(pulse_time < 0.065) {
        printf("Error: pulse time is too small to work with board\n");
        return -1;
    }
    if(trans_time < 0.065) {
        printf("Error: transient time is too small to work with board\n");
        return -1;
    }
    if(amp_val < 0.0 || amp_val > 1.0) {
        printf("Error: amplitude value out of range\n");
        return -1;
    }

    printf ("num_points  %d\n", num_points);
    printf ("SF=         %f MHz\n",SF);
    printf ("SW=         %f kHz\n", SW);
    printf ("pulse_time= %f us\n",pulse_time);
    printf ("trans_time= %f us\n",trans_time);
    printf ("repetition_delay = %f s\n", repetition_delay);
    printf ("num_scans=  %d\n",num_scans);    
    printf ("tx phase=   %f degrees\n",tx_phase);
    printf ("bypass fir= %d\n", bypass_fir);
    printf ("adc_freq=   %f MHz\n",adc_freq);
    printf ("amplitude value=   %f\n",amp_val);

    // convert to MHz
    SW /= 1000.0;

    printf ("Using spinapi library version %s\n", pb_get_version());
    if(pb_init()) {
    printf ("Error initializing board: %s\n", pb_get_error());
    return -1;
    }
    
    
    pb_set_defaults();
    pb_core_clock(adc_freq);

    pb_overflow(1,0); // reset the overflow counters
    pb_scan_count(1); // reset scan counter


	// Load the shape parameters
	shape_make_sinc(shape_data, 3);
	pb_dds_load(shape_data, DEVICE_SHAPE);
	pb_set_amp(amp_val, 0);

    ///
    /// Program frequency and phase registers
    ///
    pb_start_programming(FREQ_REGS);
    pb_set_freq(SF);
    
    while (abs(n*adc_freq/2-rxSF)>adc_freq/2)
    {
        n++;
        rxSF = (n*adc_freq/2-rxSF);
    }
    
    pb_set_freq(rxSF);
    pb_stop_programming();

    // control phases for REAL channel
    pb_start_programming(COS_PHASE_REGS);
    pb_set_phase(0.0);
    pb_set_phase(90.0);
    pb_set_phase(180.0);
    pb_set_phase(270.0);
    pb_stop_programming();

    // control phases for IMAG channel
    pb_start_programming(SIN_PHASE_REGS);
    pb_set_phase(0.0);
    pb_set_phase(90.0);
    pb_set_phase(180.0);
    pb_set_phase(270.0);
    pb_stop_programming();

    // control phases for output channel
    pb_start_programming(TX_PHASE_REGS);
    pb_set_phase(0);
    pb_set_phase(tx_phase);
    pb_stop_programming();


// User friendly names for the phase registers of the cos and sin channels
#define PHASE000 0
#define PHASE090 1
#define PHASE180 2
#define PHASE270 3

// User friendly names for the control bits
#define TX_ENABLE 1
#define TX_DISABLE 0
#define PHASE_RESET 1
#define NO_PHASE_RESET 0
#define DO_TRIGGER 1
#define NO_TRIGGER 0
#define USE_SHAPE 1
#define NO_SHAPE 0
#define AMP0 0
#define TX_PHASE 1
    ///
    /// Set acquisition parameters
    ///
    if (bypass_fir) {
        cmd = BYPASS_FIR;   
    }
    dec_amount = pb_setup_filters(SW, num_scans, cmd);
    pb_set_num_points(num_points);
   
		//actual spectral width may differ from desired spectral width (divisor of master clock oscillator frequency)
    actual_SW = (adc_freq*1e6)/(double)dec_amount;
    printf ("Actual SW used is %f kHz (desired was %f kHz)\n", actual_SW/1000.0, SW*1000.0);

    wait_time = 1000.0 * (double)num_points/actual_SW; // time in ms
    printf ("Acquisition time is %f ms\n", wait_time);

    ///
    /// Specify pulse program
    ///

//pb_inst_radio_shape (freq, cos_phase, sin_phase, tx_phase, tx_enable, phase_reset, trigger_scan, use_shape, amp, flags,
//                inst, inst_data, length)


    pb_start_programming(PULSE_PROGRAM);

	// Reset phase initially, so that the phase of the excitation pulse will be
	// the same for every scan
    pb_inst_radio_shape(0,PHASE090, PHASE000, 0, TX_DISABLE, PHASE_RESET, NO_TRIGGER, NO_SHAPE, AMP0, 0x00,
                   CONTINUE, 0, 1.0*us);

	// Create the excitation pulse (BNC0 is set to 1 here)
    if ((strcmp(shape,"Sinc")==0) || (strcmp(shape,"sinc")==0))
    {    
        start = pb_inst_radio_shape(0,PHASE090, PHASE000, TX_PHASE, TX_DISABLE, NO_PHASE_RESET, NO_TRIGGER,  USE_SHAPE, AMP0, 0x0F,
	                       LOOP, num_scans, 3*ms);
            pb_inst_radio_shape(0,PHASE090, PHASE000, TX_PHASE, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, USE_SHAPE, AMP0, 0x0F,
                   CONTINUE, 0, pulse_time * us);
    }

    else if ((strcmp(shape,"Hard")==0) || (strcmp(shape,"hard")==0))
    {
        start = pb_inst_radio_shape(0,PHASE090, PHASE000, TX_PHASE, TX_DISABLE, NO_PHASE_RESET, NO_TRIGGER,  USE_SHAPE, AMP0, 0x0F,
	                       LOOP, num_scans, 3*ms);
            pb_inst_radio_shape(0,PHASE090, PHASE000, TX_PHASE, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, NO_SHAPE, AMP0, 0x0F,
                   CONTINUE, 0, pulse_time * us);
    }
    
	// Output nothing for trans_time
    pb_inst_radio_shape(0,PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, NO_TRIGGER, NO_SHAPE, AMP0, 0x00,
                   CONTINUE, 0, trans_time * us);
	// Trigger and then wait for experiment to finish (BNC1 is set to 1 here)
    pb_inst_radio_shape(1,PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, DO_TRIGGER, NO_SHAPE, AMP0, 0x02,
                   CONTINUE, 0, wait_time * ms);
    // Now wait the repetition delay, then loop back to the beginning. Also
    // reset the phase in anticipation of the next scan
    pb_inst_radio_shape(1,PHASE090, PHASE000, 0, TX_DISABLE, PHASE_RESET, DO_TRIGGER, NO_SHAPE, AMP0, 0x00,
                   END_LOOP, start, repetition_delay*1000.0*ms);

    // Stop execution of program
    pb_inst_radio_shape(0,PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, NO_TRIGGER, NO_SHAPE, AMP0, 0x00,
                   CONTINUE, 0, 1.0*us);
    pb_inst_radio_shape(0,PHASE090, PHASE000, 0, TX_DISABLE, NO_PHASE_RESET, NO_TRIGGER, NO_SHAPE, AMP0, 0x00,
                   STOP, 0, 1.0*us);

    pb_stop_programming(); 

    ///
    /// Trigger pulse program
    ///
    pb_start();

    pb_close();
    
		printf("\n\n");
    return 0;
}



void mexFunction(int nlhs, mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
    
    int num_points;
    double SF;
    double SW;
    double pulse_time;
    double trans_time;
    double repetition_delay;
    int num_scans;
    double tx_phase;
    int bypass_fir;
    double adc_freq;
    float amp_val;
    
    char *shape;
    int shapelen;
  
    
    num_points = mxGetScalar(prhs[0]);
    SF = mxGetScalar(prhs[1]);
    SW = mxGetScalar(prhs[2]);
    pulse_time = mxGetScalar(prhs[3]);
    trans_time = mxGetScalar(prhs[4]);
    repetition_delay = mxGetScalar(prhs[5]);
    num_scans = mxGetScalar(prhs[6]);
    tx_phase = mxGetScalar(prhs[7]);
       
    bypass_fir = mxGetScalar(prhs[8]);
    adc_freq = mxGetScalar(prhs[9]);
    amp_val = mxGetScalar(prhs[10]);
    
    shapelen = (mxGetM(prhs[11]) * mxGetN(prhs[11])) + 1;
    shape = mxCalloc(shapelen, sizeof(char));
    mxGetString(prhs[11],shape,shapelen);
    
    if ((strcmp(shape,"Hard")!=0)&&(strcmp(shape,"hard")!=0)
    &&(strcmp(shape,"Sinc")!=0)&&(strcmp(shape,"sinc")!=0)) 
    {
    mexErrMsgTxt("Shape must be specified as being 'hard' or 'sinc'");
    }

    
    main(num_points, SF, SW, pulse_time, trans_time, repetition_delay,
    num_scans, tx_phase, bypass_fir, adc_freq, amp_val, shape);
    
}
